package com.cedarsoft.spring.rcp;

import org.jdesktop.swingx.VerticalLayout;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;
import org.springframework.richclient.core.UIConstants;
import org.springframework.richclient.factory.ComponentFactory;

import javax.swing.JPanel;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

/**
 * Helper object for handling several component providers.
 * It creates a panel containing all components of all providers
 */
public class ComponentProvidersSupport {
  @NotNull
  private List<ComponentProvider> componentProviders = new ArrayList<ComponentProvider>();

  /**
   * Creates a panel containing the components of the providers
   *
   * @return the panel for the set providers or null if no providers are set
   */
  @NotNull
  public JPanel create() {
    if ( componentProviders.isEmpty() ) {
      throw new IllegalStateException( "No component providers set" );
    }

    JPanel panel = new JPanel( createLayout() );

    for ( ComponentProvider provider : componentProviders ) {
      panel.add( provider.createComponent() );
    }
    return panel;
  }

  /**
   * Creates the layout that is used to layout the components returned by the component providers
   *
   * @return the layout
   */
  @NotNull
  protected VerticalLayout createLayout() {
    return new VerticalLayout( UIConstants.ONE_SPACE );
  }

  /**
   * Returns the component providers
   *
   * @return the component providers
   */
  @NotNull
  public List<? extends ComponentProvider> getComponentProviders() {
    return Collections.unmodifiableList( componentProviders );
  }

  /**
   * Adds a component provider
   *
   * @param componentProvider the component provider that is added
   */
  public void addComponentProvider( @Nullable ComponentProvider componentProvider ) {
    this.componentProviders.add( componentProvider );
  }

  /**
   * Returns whether any providers are set
   *
   * @return whether at least one provider is set
   */
  public boolean providersSet() {
    return !componentProviders.isEmpty();
  }
}
