package com.cedarsoft.spring.rcp.beans;

import com.cedarsoft.CanceledException;
import com.cedarsoft.history.ObservableObjectAccess;
import com.cedarsoft.lookup.Lookup;
import org.jetbrains.annotations.NotNull;

/**
 * The bean agent offers several actions for a single bean.
 * Those actions are UI actions and must only be called form the EDT.
 * <p/>
 * An Agent offers some sort of REST access to some kind of beans.
 *
 * @param <T> the type this agent is responsible for
 */
public interface BeanAgent<T> {
  /**
   * Creates a new bean.
   * The creation might be done using a wizard, a simple dialog, or something else.
   * The agent also might open an editor for the newly created object.
   *
   * @param objectAccess the object access
   * @param context      the context @return the new bean
   * @throws CanceledException             if the user canceled the creation of the bean
   * @throws UnsupportedOperationException if this bean agent does not support the creation of beans
   */
  void create( @NotNull ObservableObjectAccess<T> objectAccess, @NotNull Lookup context ) throws CanceledException, UnsupportedOperationException;

  /**
   * Edits the given bean.
   * Depending on the implementation of the agent the bean is edited using a wizard,
   * a dialog, a view or editor or something else.
   * <p/>
   * This call will return immediately. Changes to the bean are propagated using the listeners of the bean.
   *
   * @param objectAccess the object access
   * @param bean         the bean that is edited
   * @param context      the context
   * @throws UnsupportedOperationException if this agent does not support the editing of beans
   */
  void edit( @NotNull ObservableObjectAccess<T> objectAccess, @NotNull T bean, @NotNull Lookup context ) throws UnsupportedOperationException;
}