package com.cedarsoft.spring.rcp.binding;

import com.cedarsoft.spring.rcp.creation.CreatorModel;
import org.jetbrains.annotations.NonNls;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.NotWritablePropertyException;
import org.springframework.binding.form.FormModel;

import java.beans.PropertyDescriptor;
import java.lang.annotation.Annotation;
import java.lang.reflect.Method;

/**
 * Nullable support that is based on annotations.
 */
public class AnnotationBasedNullableSupport implements NullableSupport {
  @NotNull
  private final Class<?> nullableAnnotation;

  /**
   * Creates a new annotation support that is based on {@link Nullable}
   */
  public AnnotationBasedNullableSupport() {
    this( Nullable.class );
  }

  /**
   * Creates a new support that uses the given nullalbe annotation
   *
   * @param nullableAnnotation the nullable annotation
   */
  public AnnotationBasedNullableSupport( @NotNull Class<?> nullableAnnotation ) {
    this.nullableAnnotation = nullableAnnotation;
  }

  @Override
  public boolean isNullable( @NotNull FormModel formModel, @NotNull @NonNls String formPropertyPath ) {
    if ( formModel instanceof CreatorModel ) {
      return formModel.getValueModel( formPropertyPath ).getValue() == null;
    }

    if ( formPropertyPath.contains( "." ) ) {
      throw new IllegalArgumentException( "Deep form property paths are not supported" );
    }

    PropertyDescriptor propertyDescriptor = BeanUtils.getPropertyDescriptor( formModel.getFormObject().getClass(), formPropertyPath );
    if ( propertyDescriptor == null ) {
      throw new IllegalArgumentException( "No property descriptor found for path <" + formPropertyPath + "> at model class <" + formModel.getFormObject().getClass().getName() + '>' );
    }

    Method setter = propertyDescriptor.getWriteMethod();
    if ( setter == null ) {
      //Try fallback in case of generics
      if ( propertyDescriptor.getPropertyType().equals( Object.class ) ) {
        @NonNls
        String setterName = "set" + formPropertyPath.substring( 0, 1 ).toUpperCase() + formPropertyPath.substring( 1 );
        for ( Method method : formModel.getFormObject().getClass().getMethods() ) {
          if ( method.getParameterTypes().length == 1 && setterName.equals( method.getName() ) ) {
            setter = method;
            break;
          }
        }
      }

      if ( setter == null ) {
        throw new NotWritablePropertyException( formModel.getFormObject().getClass(), formPropertyPath );
      }
    }

    return isNullable( setter, nullableAnnotation );
  }

  public static boolean isNullable( @NotNull Method setter, @NotNull Class<?> nullableAnnotation ) {
    Annotation[] annotations = setter.getParameterAnnotations()[0];
    if ( isSet( nullableAnnotation, annotations ) ) {
      return true;
    }

    return false; //the default behaviour
  }

  public static boolean isSet( @NotNull Class<?> annotationsClass, @NotNull Annotation[] annotations ) {
    for ( Annotation annotation : annotations ) {
      if ( annotation.annotationType().equals( annotationsClass ) ) {
        return true;
      }
    }
    return false;
  }
}
