package com.cedarsoft.spring.rcp.binding;

import org.springframework.richclient.form.binding.support.CustomBinding;
import org.springframework.richclient.dialog.ApplicationDialog;
import org.springframework.richclient.dialog.CloseAction;
import org.springframework.richclient.dialog.TitledApplicationDialog;
import org.springframework.richclient.command.config.CommandConfigurer;
import org.springframework.richclient.command.ActionCommand;
import org.springframework.richclient.application.ApplicationServicesLocator;
import org.springframework.richclient.selection.binding.support.LabelProvider;
import org.springframework.binding.form.FormModel;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;
import org.jetbrains.annotations.NonNls;

import javax.swing.JTextField;
import javax.swing.JComponent;
import javax.swing.JPanel;
import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.Component;
import java.awt.Window;

import com.cedarsoft.spring.rcp.selection.NoElementSelectedException;

/**
 * Binding based on a dialog
 *
 * @param <T> the type
 */
public abstract class DialogBasedBinding<T> extends CustomBinding {
  @NotNull
  private final OpenDialogCommand openDialogCommand = new OpenDialogCommand();
  @NotNull
  private final ClearCommand clearCommand = new ClearCommand();

  @NotNull
  private final JTextField textField = new JTextField();
  private final boolean nullable;

  @NotNull
  private final LabelProvider labelProvider;

  /**
   * Creates a new dialog based binding
   *
   * @param formModel           the form model
   * @param formPropertyPath    the form property path
   * @param requiredSourceClass the required source path
   * @param labelProvider       the label provider
   * @param nullable            whether the property is nullable
   */
  protected DialogBasedBinding( @NotNull FormModel formModel, @NotNull @NonNls String formPropertyPath, @Nullable Class<T> requiredSourceClass, @NotNull LabelProvider labelProvider, boolean nullable ) {
    super( formModel, formPropertyPath, requiredSourceClass );
    this.labelProvider = labelProvider;
    this.nullable = nullable;
  }

  @Nullable
  private ApplicationDialog applicationDialog;

  @NotNull
  public ApplicationDialog getApplicationDialog() {
    if ( applicationDialog == null ) {
      applicationDialog = createApplicationDialog();
    }
    return applicationDialog;
  }

  /**
   * Creates the application dialog that is used to edit the property
   *
   * @return the application dialog
   */
  @NotNull
  protected abstract AbstractDialog createApplicationDialog();

  /**
   * The control - do not access directly
   */
  private JPanel control;

  @Override
  protected JComponent doBindControl() {
    control = new JPanel( new BorderLayout() );

    ApplicationDialog dialog = getApplicationDialog();

    dialog.setParentComponent( control );

    textField.setEditable( false );
    control.add( textField );

    // configure commands
    CommandConfigurer configurer = ( CommandConfigurer ) ApplicationServicesLocator.services().getService( CommandConfigurer.class );
    configurer.configure( openDialogCommand );
    configurer.configure( clearCommand );

    JPanel buttonPanel = new JPanel( new FlowLayout( FlowLayout.LEADING, 0, 0 ) );

    buttonPanel.add( openDialogCommand.createButton() );
    if ( nullable ) {
      buttonPanel.add( clearCommand.createButton() );
    }

    control.add( buttonPanel, BorderLayout.LINE_END );
    updateTextField();

    return control;
  }

  /**
   * Updates the content of the text field based on the current value
   */
  protected void updateTextField() {
    textField.setText( labelProvider.getLabel( getValue() ) );
  }

  @Override
  protected void valueModelChanged( Object newValue ) {
    updateTextField();
  }

  @Override
  protected void readOnlyChanged() {
    openDialogCommand.setEnabled( control.isEnabled() && !isReadOnly() );
    clearCommand.setEnabled( control.isEnabled() && !isReadOnly() );
  }

  @Override
  protected void enabledChanged() {
    control.setEnabled( isEnabled() );

    textField.setEnabled( isEnabled() );
    openDialogCommand.setEnabled( isEnabled() );
    clearCommand.setEnabled( isEnabled() );
  }

  /**
   * Clears the selected value
   */
  private class ClearCommand extends ActionCommand {
    private ClearCommand() {
      super( "dialogBasedBinding.clearCommand" );
    }

    @Override
    protected void doExecuteCommand() {
      getValueModel().setValue( null );
    }
  }

  /**
   * Opens the dialog to select the value
   */
  private class OpenDialogCommand extends ActionCommand {
    private OpenDialogCommand() {
      super( "dialogBasedBinding.openDialogCommand" );
    }

    @Override
    protected void doExecuteCommand() {
      getApplicationDialog().showDialog();
    }
  }

  /**
   * Abstract base class for dialogs.
   */
  protected abstract class AbstractDialog extends TitledApplicationDialog {
    /**
     * Creates a new dialog
     */
    protected AbstractDialog() {
    }

    /**
     * Creates a new dialog
     *
     * @param title  the title
     * @param parent the parent window
     */
    protected AbstractDialog( @NotNull String title, @Nullable Window parent ) {
      super( title, parent );

      String formModelId = getFormModel().getId();

      setTitle( getMessage( new String[]{formModelId + "." + getProperty() + ".dialog.title", getProperty() + ".dialog.title"} ) );
      setTitlePaneTitle( getMessage( new String[]{formModelId + "." + getProperty() + ".dialog.title", getProperty() + ".dialog.title"} ) );
      setDescription( getMessage( new String[]{formModelId + "." + getProperty() + ".dialog.description", getProperty() + ".dialog.description"} ) );
    }

    @Override
    protected boolean onFinish() {
      T dialogObject;
      try {
        dialogObject = getDialogObject();
      } catch ( NoElementSelectedException ignore ) { //todo check exception
        return false;
      }

      controlValueChanged( dialogObject );
      updateTextField();
      return true;
    }

    /**
     * Returns the dialog object (the selected/modified object)
     *
     * @return the dialog object
     */
    @Nullable
    protected abstract T getDialogObject();
  }
}
