package com.cedarsoft.spring.rcp.conversion;

import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;
import org.springframework.binding.convert.ConversionException;
import org.springframework.binding.convert.ConversionExecutor;
import org.springframework.binding.convert.ConversionService;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

/**
 *
 */
public class ExtendedCompositeConversionService implements ConversionService {
  @NotNull
  private final List<ConversionService> chain = new ArrayList<ConversionService>();

  @NotNull
  public List<ConversionService> getDelegates() {
    return Collections.unmodifiableList( chain );
  }

  public void setDelegates( List<ConversionService> delegates ) {
    this.chain.clear();
    this.chain.addAll( delegates );
  }

  @Override
  public ConversionExecutor getConversionExecutor( Class sourceClass, Class targetClass ) throws ConversionException {
    for ( ConversionService aChain : chain ) {
      try {
        return aChain.getConversionExecutor( sourceClass, targetClass );
      } catch ( ConversionException e ) {
        // ignore and try the next conversion service in the chain
      }
    }
    throw new ConversionException( sourceClass, targetClass, "No converter registered to convert from sourceClass '"
        + sourceClass + "' to target class '" + targetClass + "'" );
  }

  @Override
  public ConversionExecutor getConversionExecutorByTargetAlias( Class sourceClass, String targetAlias )
      throws ConversionException {
    boolean exceptionThrown = false;
    for ( ConversionService aChain : chain ) {
      try {
        ConversionExecutor res = aChain.getConversionExecutorByTargetAlias( sourceClass, targetAlias );
        if ( res != null ) {
          return res;
        }
      } catch ( ConversionException e ) {
        exceptionThrown = true;
      }
    }
    if ( exceptionThrown ) {
      throw new ConversionException( sourceClass, "No converter registered to convert from sourceClass '"
          + sourceClass + "' to aliased target type '" + targetAlias + "'" );
    } else {
      // alias was not recognized by any conversion service in the chain
      return null;
    }
  }

  @Override
  public ConversionExecutor[] getConversionExecutorsForSource( Class sourceClass ) throws ConversionException {
    Set executors = new HashSet();
    for ( ConversionService aChain : chain ) {
      executors.addAll( Arrays.asList( aChain.getConversionExecutorsForSource( sourceClass ) ) );
    }
    return ( ConversionExecutor[] ) executors.toArray( new ConversionExecutor[executors.size()] );
  }

  @Override
  @Nullable
  public Class getClassByAlias( String alias ) throws ConversionException {
    for ( ConversionService aChain : chain ) {
      try {
        Class res = aChain.getClassByAlias( alias );
        if ( res != null ) {
          return res;
        }
      } catch ( ConversionException e ) {
        // ignore and try the next conversion service in the chain
      }
    }
    return null;
  }
}
