package com.cedarsoft.spring.rcp.creation;

import com.cedarsoft.spring.rcp.PropertyPath;
import org.jetbrains.annotations.NonNls;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;
import org.springframework.binding.form.support.DefaultFormModel;
import org.springframework.binding.value.ValueModel;
import org.springframework.binding.value.support.AbstractValueModel;

/**
 * A form model that can be used to create forms for (immutable) objects.
 * <p/>
 * Call {@link #getFormObject()} after the form has been commited to get the newly created object.
 *
 * @param <T> the type
 */
public abstract class CreatorModel<T> extends DefaultFormModel {

  /**
   * Creates a new creator model for the given type
   *
   * @param id   the id
   * @param type the type
   */
  protected CreatorModel( @NotNull @NonNls String id, @NotNull Class<T> type ) {
    super( new MappedPropertyAccessStrategy( type ), true );
    setId( id );
  }

  /**
   * Registers a property that can be set
   *
   * @param propertyPath the property path
   * @param defaultValue the default value
   */
  protected final <T> void registerPropertyWithDefault( @NotNull T defaultValue, @NotNull @NonNls String... propertyPath ) {
    getFormObjectPropertyAccessStrategy().registerProperty( defaultValue, PropertyPath.toPropertyName( propertyPath ) );
  }

  protected final <T> void registerPropertyWithDefault( @NotNull T defaultValue, @NotNull PropertyPath propertyPath ) {
    getFormObjectPropertyAccessStrategy().registerProperty( defaultValue, propertyPath );
  }

  protected final <T> void registerPropertyWithDefault( @NotNull Class<? super T> type, @NotNull T defaultValue, @NotNull @NonNls String propertyName ) {
    getFormObjectPropertyAccessStrategy().registerProperty( type, defaultValue, propertyName );
  }

  /**
   * Registers a property of the given type
   *
   * @param type         the type of the property
   * @param propertyName the name of the property
   */
  protected final void registerProperty( @NotNull Class<?> type, @NotNull @NonNls String propertyName ) {
    getFormObjectPropertyAccessStrategy().registerProperty( type, propertyName );
  }

  public void setProperty( @NotNull @NonNls String propertyName, @Nullable Object value ) {
    getFormObjectPropertyAccessStrategy().getPropertyAccessor().setPropertyValue( propertyName, value );
  }

  /**
   * Returns the property value. Ensures that the property is not null.
   *
   * @param type         the type
   * @param propertyPath the path of the property
   * @return the value of the property
   */
  protected <V> V getNonNullPropertyValue( @NotNull Class<V> type, @NotNull @NonNls String... propertyPath ) {
    return getNonNullPropertyValue( type, PropertyPath.create( propertyPath ) );
  }

  protected <V> V getNonNullPropertyValue( @NotNull Class<V> type, @NotNull @NonNls PropertyPath propertyPath ) {
    V value = this.getPropertyValue( type, propertyPath );
    if ( value == null ) {
      throw new IllegalStateException( "Property " + propertyPath + " is null" );
    }
    return value;
  }

  /**
   * Returns the property value
   *
   * @param type         the type
   * @param propertyPath the property name
   * @return the value
   */
  @Nullable
  protected <V> V getPropertyValue( @NotNull Class<V> type, @NotNull @NonNls String... propertyPath ) {
    return getPropertyValue( type, PropertyPath.create( propertyPath ) );
  }

  @Nullable
  protected <V> V getPropertyValue( @NotNull Class<V> type, @NotNull @NonNls PropertyPath propertyPath ) {
    return type.cast( getValueModel( propertyPath.getProperty() ).getValue() );
  }

  @Override
  public MappedPropertyAccessStrategy getFormObjectPropertyAccessStrategy() {
    return ( MappedPropertyAccessStrategy ) super.getFormObjectPropertyAccessStrategy();
  }

  @Override
  public T getFormObject() {
    return ( T ) super.getFormObject();
  }

  @Override
  public ValueModel getFormObjectHolder() {
    return new AbstractValueModel() {
      @Override
      public Object getValue() {
        return createObject();
      }

      @Override
      public void setValue( Object newValue ) {
        throw new UnsupportedOperationException();
      }
    };
  }

  /**
   * Creates the object. This method may be called when the form has been commited.
   *
   * @return the object that has been created
   */
  @NotNull
  protected abstract T createObject();
}
