package com.cedarsoft.spring.rcp.forms;

import com.cedarsoft.spring.rcp.TabProvider;
import org.jetbrains.annotations.NonNls;
import org.jetbrains.annotations.NotNull;
import org.springframework.binding.form.FormModel;
import org.springframework.binding.form.ValidatingFormModel;
import org.springframework.richclient.dialog.AbstractDialogPage;
import org.springframework.richclient.dialog.TabbedDialogPage;
import org.springframework.richclient.form.AbstractForm;
import org.springframework.richclient.form.FormModelHelper;
import org.springframework.richclient.form.binding.swing.SwingBindingFactory;
import org.springframework.richclient.form.builder.TableFormBuilder;

import javax.swing.JComponent;
import javax.swing.JLabel;

/**
 * Base class for all forms within avr
 *
 * @param <T> the type
 */
public abstract class ExtendedForm<T> extends AbstractForm {
  @NonNls
  public static final String COL_SPAN_1 = "colSpan=1";
  @NonNls
  public static final String COL_LEFT_PREF = "colSpec=left:pref";
  @NonNls
  public static final String COL_LEFT_PREF_GROW = "colSpec=fill:pref:grow(1)";
  @NonNls
  public static final String ROW_TOP_DEFAULT = "rowSpec=top:default";
  @NonNls
  public static final String ROW_PREF_GROW = "rowSpec=fill:pref:grow(1)";
  @NonNls
  public static final String ROW_MIN_GROW = "rowSpec=fill:min:grow(1)";
  @NonNls
  protected static final String COL_LABEL = "colGrId=label colSpec=right:pref";

  protected ExtendedForm( @NotNull @NonNls String formId, @NotNull T formObject ) {
    super( FormModelHelper.createFormModel( formObject, formId ) );
    setId( formId );
  }

  /**
   * Creates a new form
   *
   * @param formModel the form model
   */
  protected ExtendedForm( @NotNull FormModel formModel ) {
    super( formModel );
    assert formModel.getId() != null;
  }

  /**
   * The form model should have an id, too
   *
   * @param formId    the form id
   * @param formModel the form model
   */
  protected ExtendedForm( @NotNull @NonNls String formId, @NotNull FormModel formModel ) {
    super( formModel, formId );
  }

  @Override
  public void commit() {
    T formObject = getFormObject();
    synchronized ( formObject ) { //Synchronization necessary because of terracotta...
      //todo add support for lock provider(?!)
      super.commit();
    }
  }

  @Override
  @NotNull
  public final JComponent createFormControl() {
    TableFormBuilder builder = createFormBuilder();
    configureForm( builder );
    return builder.getForm();
  }

  /**
   * Creates a new table form builder
   *
   * @return a new table form builder
   */
  @NotNull
  protected TableFormBuilder createFormBuilder() {
    TableFormBuilder builder = new TableFormBuilder( getBindingFactory() );
    configure( builder );
    return builder;
  }

  /**
   * Configures a table form builder
   *
   * @param builder the builder
   */
  protected void configure( @NotNull TableFormBuilder builder ) {
    builder.setLabelAttributes( COL_LABEL );
  }

  /**
   * Binds each of the given propertis within one tab
   *
   * @param builder    the builder
   * @param tabId      the tab id
   * @param properties the properties that are bound within the tabs
   */
  protected void bindInTabs( @NotNull TableFormBuilder builder, @NotNull @NonNls String tabId, @NotNull @NonNls String... properties ) {
    bindInTabs( "", builder, tabId, properties );
  }

  /**
   * Binds each of the given properties within one tab
   *
   * @param attributes the attributes
   * @param builder    the builder
   * @param tabId      the id of the tabbed pane
   * @param properties the properties that are bound within the tabs
   */
  protected void bindInTabs( @NotNull @NonNls String attributes, @NotNull TableFormBuilder builder, @NotNull @NonNls String tabId, @NotNull @NonNls String... properties ) {
    if ( properties.length == 0 ) {
      throw new IllegalStateException( "Need at least one property" );
    }

    TabbedDialogPage tabbedDialogPage = new TabbedDialogPage( tabId );
    for ( final String property : properties ) {
      tabbedDialogPage.addPage( new AbstractDialogPage( property, false ) {
        @Override
        protected JComponent createControl() {
          return getBindingFactory().createBinding( property ).getControl();
        }
      } );
    }

    builder.getLayoutBuilder().cell( tabbedDialogPage.getControl(), attributes );
  }

  protected void bindInTabs( @NotNull TableFormBuilder builder, @NotNull @NonNls String tabId, @NotNull @NonNls TabProvider... tabProviders ) {
    bindInTabs( "", builder, tabId, tabProviders );
  }

  protected void bindInTabs( @NotNull @NonNls String attributes, @NotNull TableFormBuilder builder, @NotNull @NonNls String tabId, @NotNull @NonNls TabProvider... tabProviders ) {
    if ( tabProviders.length == 0 ) {
      throw new IllegalStateException( "Need at least one tab configuration" );
    }

    TabbedDialogPage tabbedDialogPage = new TabbedDialogPage( tabId );
    for ( TabProvider tabProvider : tabProviders ) {
      tabbedDialogPage.addPage( tabProvider.createTab() );
    }

    builder.getLayoutBuilder().cell( tabbedDialogPage.getControl(), attributes );
  }

  @Override
  public T getFormObject() {
    return ( T ) super.getFormObject();
  }

  protected <S> void bindSubForm( @NotNull TableFormBuilder builder, @NotNull @NonNls String property, @NotNull FormFactory<S> factory ) {
    bindSubForm( builder, property, factory, true );
  }

  protected <S> void bindSubForm( @NotNull TableFormBuilder builder, @NotNull @NonNls String property, @NotNull FormFactory<S> factory, boolean buffered ) {
    ValidatingFormModel childModel = FormModelHelper.createFormModel( getFormModel().getValueModel( property ), buffered, null );
    getFormModel().addChild( childModel );

    ExtendedForm<? extends S> childForm = factory.createForm( childModel );
    addChildForm( childForm );

    builder.getLayoutBuilder().cell( childForm.createFormControl() );
  }

  protected <S> void bindSubForm( @NotNull TableFormBuilder builder, @NotNull @NonNls String property, @NotNull final Class<? extends ExtendedForm<S>> formClass ) {
    bindSubForm( builder, property, formClass, true );
  }

  protected <S> void bindSubForm( @NotNull TableFormBuilder builder, @NotNull @NonNls String property, @NotNull final Class<? extends ExtendedForm<S>> formClass, boolean buffered ) {
    bindSubForm( builder, property, ReflectionFormFactory.create( formClass ), buffered );
  }

  /**
   * @param propertyReference the property reference
   * @return the label
   *
   * @deprecated use the method of form builder instead
   */
  @Deprecated
  @NotNull
  protected JLabel createLabelFor( @NotNull @NonNls String propertyReference ) {
    JLabel label = getComponentFactory().createLabel( "" );
    getFormModel().getFieldFace( propertyReference ).configure( label );
    return label;
  }

  @Override
  public SwingBindingFactory getBindingFactory() {
    return ( SwingBindingFactory ) super.getBindingFactory();
  }

  /**
   * Configure the form
   *
   * @param builder the builder (a table form builder)
   */
  protected abstract void configureForm( @NotNull @NonNls TableFormBuilder builder );
}
