package com.cedarsoft.spring.rcp.hierarchy;

import com.cedarsoft.spring.rcp.events.UiUpdateEvent;
import com.cedarsoft.utils.ChildChangeListener;
import com.cedarsoft.utils.ChildDetector;
import com.cedarsoft.utils.ParentCache;
import com.cedarsoft.utils.Renderer;
import com.cedarsoft.utils.TypeRegistry;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;
import org.springframework.context.ApplicationEvent;
import org.springframework.richclient.application.event.LifecycleApplicationEvent;

import javax.swing.SwingUtilities;
import javax.swing.tree.TreeCellRenderer;
import javax.swing.tree.TreePath;
import java.util.Collections;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Queue;
import java.util.WeakHashMap;

/**
 * Special tree model that is based on child detectors.
 */
public class HierachyTreeModel extends AbstractTreeModel {
  @NotNull
  private static final Log log = LogFactory.getLog( HierachyTreeModel.class );

  @Deprecated
  @NotNull
  public static final Root ROOT = new Root();

  @NotNull
  private final TypeRegistry<HierarchyLayer<?, ?>> registry;
  @NotNull
  private final ManagedTreeCellRenderer renderer;
  @NotNull
  private final ParentCache parentCache = new ParentCache();

  @NotNull
  private final Object root;

  public HierachyTreeModel( @NotNull Object root, @NotNull Renderer<Object, Object> fallbackRenderer, @NotNull TypeRegistry<HierarchyLayer<?, ?>> hierarchyLayerRegistry ) {
    this.root = root;
    registry = hierarchyLayerRegistry;
    renderer = new ManagedTreeCellRenderer( fallbackRenderer ) {
      @Override
      @Nullable
      protected Renderer<Object, Object> getRenderer( @NotNull Object value ) throws IllegalArgumentException {
        HierarchyLayer<Object, Object> layer = getLayer( value );
        if ( layer == null ) {
          return null;
        }
        //noinspection unchecked
        return layer.getLookup().lookup( Renderer.class );
      }
    };
  }

  @Override
  @NotNull
  public Object getRoot() {
    return root;
  }

  /**
   * Returns the child detector for the given parent
   *
   * @param parent the parent
   * @return the child detector
   *
   * @throws ChildDetectorNotFoundException
   */
  @Nullable
  protected <P, C> ChildDetector<P, C> getChildDetector( @NotNull P parent ) throws ChildDetectorNotFoundException {
    HierarchyLayer<P, C> layer = this.getLayer( parent );
    if ( layer == null ) {
      return null;
    }
    //noinspection unchecked
    ChildDetector<P, C> childDetector = layer.getLookup().lookup( ChildDetector.class );
    if ( childDetector != null ) {
      //register listener
      if ( childDetectorListeners.get( childDetector ) == null ) {
        //noinspection AnonymousClassVariableHidesContainingMethodVariable
        ChildChangeListener<P> listener = new ChildChangeListener<P>() {
          @Override
          public void notifyChildrenChangedFor( @NotNull final P parent ) {
            SwingUtilities.invokeLater( new Runnable() {
              @Override
              public void run() {
                handleModified( parent );
              }
            } );
          }
        };
        childDetector.addChangeListener( listener );
        childDetectorListeners.put( childDetector, listener );
      }
    }
    return childDetector;
  }

  private final Map<ChildDetector<?, ?>, ChildChangeListener<?>> childDetectorListeners = new WeakHashMap<ChildDetector<?, ?>, ChildChangeListener<?>>();

  /**
   * Returns the HierarchyLayer for the given parent
   *
   * @param parent the parent
   * @return the hierarchy layer
   *
   * @throws IllegalArgumentException
   */
  @Nullable
  protected <P, C> HierarchyLayer<P, C> getLayer( @NotNull P parent ) throws IllegalArgumentException {
    return ( HierarchyLayer<P, C> ) registry.getElement( parent.getClass() );
  }

  @NotNull
  protected <P, C> List<? extends C> findChildren( @NotNull P parent ) {
    try {
      ChildDetector<P, C> detector = this.getChildDetector( parent );
      if ( detector == null ) {
        return Collections.emptyList();
      }
      List<? extends C> children = detector.findChildren( parent );
      fillParentCache( parent, children );
      return children;
    } catch ( ChildDetectorNotFoundException ignore ) {
      return Collections.emptyList();
    }
  }

  private void fillParentCache( @NotNull Object parent, @NotNull List<? extends Object> children ) {
    parentCache.store( parent, children );
  }

  @Override
  public Object getChild( Object parent, int index ) {
    return findChildren( parent ).get( index );
  }

  @Override
  public int getChildCount( Object parent ) {
    return findChildren( parent ).size();
  }

  @Override
  public boolean isLeaf( Object node ) {
    try {
      return getChildDetector( node ) == null;
    } catch ( ChildDetectorNotFoundException ignore ) {
      return true;
    }
  }

  @Override
  public int getIndexOfChild( Object parent, Object child ) {
    return findChildren( parent ).indexOf( child );
  }

  @Override
  public void valueForPathChanged( TreePath path, Object newValue ) {
  }

  private boolean updating = true;

  public void handle( @NotNull ApplicationEvent event ) {
    //Wait for update notifications
    if ( event instanceof UiUpdateEvent ) {
      UiUpdateEvent.Type type = ( ( UiUpdateEvent ) event ).getType();
      updating = type == UiUpdateEvent.Type.ACTIVATE_UPDATES;
      log.debug( "Switched updating to " + updating );

      if ( type == UiUpdateEvent.Type.ACTIVATE_UPDATES ) {
        log.debug( "Loading all changes" );
        fireRootTreeStructureChanged( root );
      }
    }

    if ( !updating ) {
      return;
    }

    if ( !( event instanceof LifecycleApplicationEvent ) ) {
      return;
    }

    LifecycleApplicationEvent lifecycleEvent = ( LifecycleApplicationEvent ) event;

    Object object = lifecycleEvent.getObject();
    if ( lifecycleEvent.getEventType().equals( LifecycleApplicationEvent.MODIFIED ) ) {
      handleModified( object );
    } else if ( lifecycleEvent.getEventType().equals( LifecycleApplicationEvent.DELETED ) ) {
      handleDeleted( object );
    } else if ( lifecycleEvent.getEventType().equals( LifecycleApplicationEvent.CREATED ) ) {
      handleCreated( object );
    }
  }

  /**
   * Guess the parent for the given child, that has (maybe) not been added to the parent cache
   *
   * @param newChild the new child
   * @return the parent
   */
  @NotNull
  protected Object guessParent( @NotNull Object newChild ) {
    {
      Object parent = parentCache.findParent( newChild );
      if ( parent != null ) {
        log.debug( "Didn't have to guess - found parent!" );
        return parent;
      }
    }

    //todo Performance

    //Traverse all until found
    Queue<Object> queue = new LinkedList<Object>();
    queue.add( getRoot() );
    while ( !queue.isEmpty() ) {
      Object current = queue.poll();
      log.debug( "Searching children for " + current );
      queue.addAll( findChildren( current ) );

      Object parent = parentCache.findParent( newChild );
      if ( parent != null ) {
        return parent;
      }
    }
    throw new IllegalArgumentException( "Impossible - didn't find a parent for " + newChild );
  }

  private void handleCreated( @NotNull Object object ) {
    log.debug( "Handling created object " + object );
    try {
      Object parent = guessParent( object );
      fireTreeNodeInserted( findPath( parent ), findIndex( object ), object );
    } catch ( IllegalArgumentException ignore ) {//not in tree
    }
  }

  private void handleDeleted( @NotNull Object object ) {
    log.debug( "Handling deleted object " + object );
    try {
      fireTreeNodeRemoved( findParentPath( object ), findIndex( object ), object );
    } catch ( IllegalArgumentException ignore ) {//not in tree
    }
  }

  private void handleModified( Object object ) {
    log.debug( "Handling modified object " + object );
    try {
      fireTreeNodeChanged( findParentPath( object ), findIndex( object ), object );
      fireTreeStructureChanged( findParentPath( object ), findIndex( object ), object );
    } catch ( IllegalArgumentException ignore ) {//not in tree
    }
  }

  private int findIndex( @NotNull Object child ) {
    Integer index = parentCache.findIndex( child );
    if ( index == null ) {
      throw new IllegalArgumentException( "No index found for " + child );
    }
    return index;
  }

  @NotNull
  private Object findParent( @NotNull Object child ) {
    Object parent = parentCache.findParent( child );
    if ( parent == null ) {
      throw new IllegalArgumentException( "No parent found for child " + child );
    }
    return parent;
  }

  /**
   * Returns the tree path for the given object
   *
   * @param object the object
   * @return the path for the object
   */
  @NotNull
  public Object[] findPath( @NotNull Object object ) {
    return parentCache.getPath( object ).toArray();
  }

  /**
   * Returns the tree path for the parent of the given object
   *
   * @param object the object
   * @return the path for the parent of the object
   */
  @NotNull
  protected Object[] findParentPath( @NotNull Object object ) {
    return parentCache.getPredecessors( object ).toArray();
  }

  @NotNull
  public TreeCellRenderer getRenderer() {
    return renderer;
  }

  @Deprecated
  public static final class Root {
    private Root() {
    }
  }
}
