package com.cedarsoft.spring.rcp.table;

import com.cedarsoft.spring.rcp.PropertyPath;
import org.jetbrains.annotations.NonNls;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableColumn;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;

/**
 * Defines a column
 */
public class ColumnDefinition {
  @NotNull
  public static ColumnDefinition create( @NotNull @NonNls String... propertyPath ) {
    return create( 0, propertyPath );
  }

  @NotNull
  public static ColumnDefinition create( int width, @NotNull @NonNls String... propertyPath ) {
    return create( PropertyPath.create( propertyPath ), width );
  }

  @NotNull
  public static ColumnDefinition create( @Nullable TableCellRenderer cellRenderer, int width, @NotNull @NonNls String... propertyPath ) {
    return create( PropertyPath.create( propertyPath ), width, cellRenderer );
  }

  @NotNull
  public static ColumnDefinition create( @Nullable Comparator<?> comparator, @Nullable TableCellRenderer cellRenderer, int width, @NotNull @NonNls String... propertyPath ) {
    return create( PropertyPath.create( propertyPath ), width, cellRenderer, comparator );
  }

  @NotNull
  public static ColumnDefinition create( @Nullable Comparator<?> comparator, int width, @NotNull @NonNls String... propertyPath ) {
    return create( PropertyPath.create( propertyPath ), width, null, comparator );
  }

  @NotNull
  private static ColumnDefinition create( @NotNull PropertyPath propertyPath, int width ) {
    return create( propertyPath, width, null );
  }

  @NotNull
  private static ColumnDefinition create( @NotNull PropertyPath propertyPath, int width, @Nullable TableCellRenderer cellRenderer ) {
    return create( propertyPath, width, cellRenderer, null );
  }

  @NotNull
  private static ColumnDefinition create( @NotNull PropertyPath propertyPath, int width, @Nullable TableCellRenderer cellRenderer, @Nullable Comparator<?> comparator ) {
    ColumnDefinition definition = new ColumnDefinition( propertyPath );
    definition.setWidth( width );
    definition.setCellRenderer( cellRenderer );
    definition.setComparator( comparator );
    return definition;
  }

  @NotNull
  @NonNls
  private final PropertyPath propertyPath;

  @Nullable
  @NonNls
  private String columnNameOverride;
  private int width;
  @Nullable
  private TableCellRenderer cellRenderer;

  @Nullable
  private Comparator<?> comparator;

  /**
   * Creates a new column definition
   *
   * @param propertyPath the property path for this column
   */
  public ColumnDefinition( @NonNls @NotNull PropertyPath propertyPath ) {
    this.propertyPath = propertyPath;
  }

  /**
   * Sets the custom cell renderer
   *
   * @param cellRenderer the cell renderer
   */
  public void setCellRenderer( @Nullable TableCellRenderer cellRenderer ) {
    this.cellRenderer = cellRenderer;
  }

  @Nullable
  public TableCellRenderer getCellRenderer() {
    return cellRenderer;
  }

  @NotNull
  public PropertyPath getPropertyPath() {
    return propertyPath;
  }

  public void setWidth( int width ) {
    this.width = width;
  }

  public int getWidth() {
    return width;
  }

  public void setComparator( @Nullable Comparator<?> comparator ) {
    this.comparator = comparator;
  }

  @Nullable
  public Comparator<?> getComparator() {
    return comparator;
  }

  /**
   * Applies the settings to the given column
   *
   * @param column the column
   */
  public void apply( @NotNull TableColumn column ) {
    //Apply the width
    if ( width > 0 ) {
      column.setPreferredWidth( width );
    }

    //Apply the renderer
    if ( cellRenderer != null ) {
      column.setCellRenderer( cellRenderer );
    }
  }

  /**
   * Returns an array containing all property names of the given column definitions.
   *
   * @param columnDefinitions the column definitions
   * @return the property names
   */
  @NonNls
  @NotNull
  public static String[] extractColumnProperties( @NotNull List<? extends ColumnDefinition> columnDefinitions ) {
    String[] properties = new String[columnDefinitions.size()];

    for ( int i = 0; i < columnDefinitions.size(); i++ ) {
      ColumnDefinition definition = columnDefinitions.get( i );
      properties[i] = definition.getPropertyPath().getProperty();
    }

    return properties;
  }

  @NotNull
  public static List<? extends ColumnDefinition> createDefinitions( String[] columnPropertyNames ) {
    List<ColumnDefinition> definitions = new ArrayList<ColumnDefinition>();

    for ( String columnPropertyName : columnPropertyNames ) {
      definitions.add( ColumnDefinition.create( columnPropertyName ) );
    }

    return definitions;
  }

  @Nullable
  @NonNls
  public String getColumnNameOverride() {
    return columnNameOverride;
  }

  public void setColumnNameOverride( @Nullable @NonNls String columnNameOverride ) {
    this.columnNameOverride = columnNameOverride;
  }
}
