package com.cedarsoft.spring.rcp.tbpanel.aspects;

import com.cedarsoft.history.ObservableObjectAccess;
import com.cedarsoft.spring.rcp.beans.BeanAgent;
import com.cedarsoft.spring.rcp.commands.CommandBuilder;
import com.cedarsoft.spring.rcp.commands.CommandIds;
import com.cedarsoft.spring.rcp.commands.CommandProvider;
import com.cedarsoft.spring.rcp.table.ObjectTable;
import com.cedarsoft.spring.rcp.tbpanel.ObjectTablePanel;
import com.cedarsoft.spring.rcp.tbpanel.SidePanelCommandsProvider;
import com.cedarsoft.spring.rcp.tbpanel.config.DoubleClickConfigurer;
import com.cedarsoft.spring.rcp.tbpanel.config.PopupConfigurer;
import org.jetbrains.annotations.NotNull;
import org.springframework.binding.value.ValueModel;
import org.springframework.richclient.application.PageComponentContext;
import org.springframework.richclient.command.ActionCommand;
import org.springframework.richclient.command.support.GlobalCommandIds;
import org.springframework.richclient.list.ListSelectionValueModelAdapter;
import org.springframework.richclient.list.ListSingleSelectionGuard;

import javax.swing.JPanel;
import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.List;

/**
 * Aspect that adds the possibility to edit objects
 *
 * @param <T> the type
 */
public class EditableAspect<T> implements ObjectTablePanelAspect<T> {
  /**
   * Factory method
   *
   * @param agent        the agent
   * @param objectAccess the object access
   * @param parent       the parent
   * @return the created add/remove aspect
   */
  @NotNull
  public static <T> EditableAspect<T> create( @NotNull BeanAgent<T> agent, @NotNull ObservableObjectAccess<T> objectAccess, @NotNull JPanel parent ) {
    //Add/Remove
    return new EditableAspect<T>( new DefaultEditor<T>( agent, objectAccess, parent ) );
  }

  @NotNull
  private final CommandBuilder<ObjectTablePanel<T>> editCommandBuilder = new CommandBuilder<ObjectTablePanel<T>>();

  /**
   * Creates a new editable aspect
   *
   * @param editor the editor
   */
  public EditableAspect( @NotNull final Editor<T> editor ) {
    this( new CommandProvider<ObjectTablePanel<T>>() {
      @NotNull
      @Override
      public ActionCommand getCommand( @NotNull final ObjectTablePanel<T> object ) {
        return new ActionCommand( CommandIds.EDIT ) {
          @Override
          protected void doExecuteCommand() {
            ObjectTable<T> objectTable = object.getObjectTable();

            T element = objectTable.getSelectedObject();
            editor.editEntry( element );
          }
        };
      }
    } );
  }

  /**
   * Creates a new editable aspect
   *
   * @param editCommandProvider the edit command provider
   */
  public EditableAspect( @NotNull CommandProvider<ObjectTablePanel<T>> editCommandProvider ) {
    getEditCommandBuilder().setProvider( editCommandProvider );
  }

  @Override
  public void apply( @NotNull final ObjectTablePanel<T> panel ) {
    final ActionCommand editCommand = getEditCommandBuilder().getCommand( panel );
    if ( editCommand == null ) {
      return;
    }

    //add the edit command as double click command
    panel.addConfigurer( new DoubleClickConfigurer<T>( new CommandProvider<ObjectTable<T>>() {
      @Override
      @NotNull
      public ActionCommand getCommand( @NotNull ObjectTable<T> object ) {
        return editCommand;
      }
    } ) );


    panel.addConfigurer( new PopupConfigurer<T>() {
      @Override
      @NotNull
      protected List<? extends ActionCommand> getCommands( @NotNull ObjectTable<T> objectTable ) {
        return Arrays.asList( editCommand );
      }
    } );

    panel.addCommandProvider( new SidePanelCommandsProvider() {
      @Override
      @NotNull
      public Collection<? extends ActionCommand> getCommands( @NotNull ValueModel selectionHolder ) {
        ObjectTable<T> objectTable = panel.getObjectTable();

        //noinspection ResultOfObjectAllocationIgnored
        new ListSingleSelectionGuard( new ListSelectionValueModelAdapter( objectTable.getSelectionModel() ), editCommand );
        return Collections.singletonList( editCommand );
      }

      @Override
      public void registerLocalCommandExecutors( @NotNull PageComponentContext context ) {
        context.register( GlobalCommandIds.PROPERTIES, editCommand );
      }
    } );
  }

  @NotNull
  public final CommandBuilder<ObjectTablePanel<T>> getEditCommandBuilder() {
    return editCommandBuilder;
  }

  /**
   * The editor callback
   *
   * @param <T> the type
   */
  public interface Editor<T> {
    /**
     * Edits an object
     *
     * @param selectedObject the selected object
     */
    void editEntry( @NotNull T selectedObject );
  }
}
