package com.cedarsoft.spring.rcp.tbpanel.aspects;

import com.cedarsoft.lookup.Lookup;
import com.cedarsoft.spring.SpringSupport;
import com.cedarsoft.spring.rcp.ComponentProvider;
import com.cedarsoft.spring.rcp.selection.SelectionChangeListener;
import com.cedarsoft.spring.rcp.selection.SelectionModel;
import com.cedarsoft.spring.rcp.tbpanel.ObjectTablePanel;
import com.jidesoft.swing.StyleRange;
import com.jidesoft.swing.StyledLabel;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;
import org.springframework.richclient.selection.binding.support.LabelProvider;

import javax.swing.JComponent;
import java.awt.Font;

/**
 * Based on a selection aware access this
 *
 * @param <T> type of the table based panel
 * @param <S> the type of the selection
 */
public class SelectionAwareHeaderAspect<T, S> implements ObjectTablePanelAspect<T> {
  @NotNull
  private final Class<S> selectionType;
  @NotNull
  private final LabelProvider labelProvider;

  @NotNull
  private final SelectionModel selectionModel;

  /**
   * Creates a new header aspect
   *
   * @param selectionType  the selection type
   * @param labelProvider  the label provider
   * @param selectionModel
   */
  public SelectionAwareHeaderAspect( @NotNull Class<S> selectionType, @NotNull LabelProvider labelProvider, @NotNull SelectionModel selectionModel ) {
    this.selectionType = selectionType;
    this.labelProvider = labelProvider;
    this.selectionModel = selectionModel;
  }

  @Override
  public void apply( @NotNull final ObjectTablePanel<T> panel ) {
    panel.addHeadComponentProvider( new SelectionBasedComponentProvider<S>( selectionType, labelProvider, selectionModel ) );
  }

  /**
   * Returns a JLabel based on the current selection of a given type.
   *
   * @param <S> the type of the selected object
   */
  public static class SelectionBasedComponentProvider<S> implements ComponentProvider {
    @NotNull
    private final StyledLabel label;
    @NotNull
    private final SelectionModel selectionModel;
    @NotNull
    private final LabelProvider labelProvider;

    /**
     * Creates a new provider.
     *
     * @param selectionType  the selection type
     * @param labelProvider  the label provider
     * @param selectionModel the selection model
     */
    public SelectionBasedComponentProvider( @NotNull final Class<S> selectionType, @NotNull LabelProvider labelProvider, @NotNull SelectionModel selectionModel ) {
      this.labelProvider = labelProvider;
      label = new StyledLabel();

      //update the selection
      this.selectionModel = selectionModel;
      selectionModel.addSelectionChangeListener( new SelectionChangeListener() {
        @Override
        public void selectionChanged( @NotNull Lookup selectedObjects ) {
          S selected = selectedObjects.lookup( selectionType );
          updateLabel( selected );
        }
      } );
      updateLabel( selectionModel.getCurrentlySelectedObject().lookup( selectionType ) );

      label.addStyleRange( new StyleRange( Font.ITALIC ) );
    }

    @NotNull
    public SelectionModel getSelectionModel() {
      return selectionModel;
    }

    @Override
    @NotNull
    public JComponent createComponent() {
      return label;
    }

    private void updateLabel( @Nullable S selected ) {
      String text;
      if ( selected == null ) {
        text = SpringSupport.INSTANCE.getMessage( "based.on.empty.selection" );
      } else {
        String selectedObjectLabel = labelProvider.getLabel( selected );
        text = SpringSupport.INSTANCE.getMessage( "based.on.selection", selectedObjectLabel );
      }

      label.setText( text );
    }
  }
}
