package com.cedarsoft.spring.rcp.tree;

import com.cedarsoft.SwingHelper;
import com.cedarsoft.history.ElementsListener;
import com.cedarsoft.history.ObservableCollection;
import com.cedarsoft.history.ObservableObjectAccess;
import com.cedarsoft.history.SingleElementsListener;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import javax.swing.tree.DefaultMutableTreeNode;
import java.util.Map;
import java.util.WeakHashMap;
import java.util.concurrent.locks.ReadWriteLock;

/**
 * This instance is a bridge between an {@link ExtendedTreeModel} and
 * {@link ObservableObjectAccess}s.
 * <p/>
 * Changes to the object access are mirrored to the tree.
 */
public class ObservableCollectionBridge<T> extends SingleElementsListener<T> {
  @NotNull
  protected final ExtendedTreeModel model;
  @NotNull
  private final DefaultMutableTreeNode parentNode;
  @Nullable
  private final Callback<T> callback;

  public ObservableCollectionBridge( @NotNull ExtendedTreeModel model, @NotNull DefaultMutableTreeNode parentNode, @Nullable Callback<T> callback ) {
    this.model = model;
    this.parentNode = parentNode;
    this.callback = callback;
  }

  @Override
  public void elementDeleted( @NotNull ObservableCollection<? extends T> source, @NotNull final T element, int index ) {
    SwingHelper.early( new Runnable() {
      @Override
      public void run() {
        DefaultMutableTreeNode node = findChild( element );
        model.removeNodeFromParent( node );
        handleDeleted( element, listeners.get( element ) );
      }
    } );
  }

  protected void storeListener( @NotNull T element, @NotNull ElementsListener<?> listener ) {
    listeners.put( element, listener );
  }

  @NotNull
  protected ElementsListener<?> getListener( @NotNull T element ) {
    ElementsListener<?> listener = listeners.get( element );
    if ( listener == null ) {
      throw new IllegalArgumentException( "No listener found for " + element );
    }
    return listener;
  }

  @Override
  public void elementAdded( @NotNull ObservableCollection<? extends T> source, @NotNull final T element, final int index ) {
    SwingHelper.early( new Runnable() {
      @Override
      public void run() {
        DefaultMutableTreeNode node = model.addChild( parentNode, element, index );

        ElementsListener<?> listener = handleAddition( element, node );
        if ( listener != null ) {
          storeListener( element, listener );
        }
      }
    } );
  }

  @Override
  public void elementChanged( @NotNull ObservableCollection<? extends T> source, @NotNull final T element, int index ) {
    SwingHelper.early( new Runnable() {
      @Override
      public void run() {
        model.nodeChanged( findChild( element ) );
      }
    } );
  }

  @NotNull
  protected DefaultMutableTreeNode findChild( @NotNull T element ) {
    return TreeUtils.findNode( parentNode, element );
  }

  /**
   * Handles the addition of the given element
   *
   * @param element the element that is added
   * @param node    the node
   * @return an element listener that is stored for the given element
   */
  @Nullable
  protected ElementsListener<?> handleAddition( @NotNull T element, @NotNull DefaultMutableTreeNode node ) {
    if ( callback != null ) {
      return callback.handleAddition( element, node );
    } else {
      return null;
    }
  }

  /**
   * Handles the deletion of the
   *
   * @param element        the element
   * @param storedListener the stored listener
   */
  protected void handleDeleted( @NotNull T element, @Nullable ElementsListener<?> storedListener ) {
    if ( callback != null ) {
      callback.handleDeleted( element, storedListener );
    }
  }

  @NotNull
  private final Map<T, ElementsListener<?>> listeners = new WeakHashMap<T, ElementsListener<?>>();

  /**
   * Installs the bridge to the given collection
   *
   * @param observableCollection the collection
   */
  public void install( @NotNull ObservableCollection<T> observableCollection ) {
    ReadWriteLock lock = observableCollection.getLock();
    lock.readLock().lock();
    try {
      for ( int index = 0; index < observableCollection.getElements().size(); index++ ) {
        T element = observableCollection.getElements().get( index );
        elementAdded( observableCollection, element, index );
      }

      observableCollection.addElementListener( this );
    } finally {
      lock.readLock().unlock();
    }
  }

  /**
   * A callback for the next layer
   *
   * @param <T> the type
   */
  public interface Callback<T> {
    /**
     * Handles the addition of an elements
     *
     * @param element the element
     * @param node    the node
     * @return the listener (or null)
     */
    @Nullable
    ElementsListener<?> handleAddition( @NotNull T element, @NotNull DefaultMutableTreeNode node );

    /**
     * Handles the deletion of an element
     *
     * @param element        the element
     * @param storedListener the stored listener
     */
    void handleDeleted( @NotNull T element, @Nullable ElementsListener<?> storedListener );
  }
}
