package com.cedarsoft.utils.springrcp.beanlist;

import com.cedarsoft.utils.springrcp.renderer.EnumCellRenderer;
import com.cedarsoft.utils.springrcp.renderer.Rectangle2DTableCellRenderer;
import com.cedarsoft.utils.springrcp.selection.DefaultSelection;
import com.cedarsoft.utils.springrcp.selection.Selection;
import com.cedarsoft.utils.springrcp.selection.SelectionManager;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;
import org.springframework.richclient.application.support.AbstractView;
import org.springframework.richclient.command.CommandGroup;
import org.springframework.richclient.core.UIConstants;
import org.springframework.richclient.layout.TableLayoutBuilder;
import org.springframework.richclient.table.BaseTableModel;
import org.springframework.richclient.table.TableUtils;
import org.springframework.richclient.util.GuiStandardUtils;
import org.springframework.richclient.util.PopupMenuMouseListener;

import javax.swing.BorderFactory;
import javax.swing.JComponent;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.ListSelectionModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.table.TableModel;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.geom.Rectangle2D;

/**
 * This is a very basic class for a view that shows beans in a list
 * <p/>
 * Date: 29.08.2006<br>
 * Time: 13:11:41<br>
 *
 * @author <a href="http://johannes-schneider.info">Johannes Schneider</a> -
 *         <a href="http://www.xore.de">Xore Systems</a>
 */
public abstract class AbstractBeanListView<T> extends AbstractView {
  private JTable table;

  @Override
  protected JComponent createControl() {
    initTable();
    initSelectionChanges();

    TableLayoutBuilder builder = new TableLayoutBuilder();
    JScrollPane scrollPane = new JScrollPane( getTable() );

    builder.row();
    builder.cell( scrollPane, "colSpec=fill:default:grow rowSpec=fill:default:grow" );

    scrollPane.getViewport().setOpaque( true );
    scrollPane.getViewport().setBackground( getTable().getBackground() );

    builder.row();
    builder.cell( createDefaultCommandGroup().createButtonBar() );

    initializeCommands();

    finalizeInitialization();
    return GuiStandardUtils.attachBorder( builder.getPanel(), BorderFactory.createEmptyBorder( UIConstants.ONE_SPACE, UIConstants.ONE_SPACE, 0, UIConstants.ONE_SPACE ) );
  }

  protected abstract void initializeCommands();

  /**
   * Is called at the end of the create controll process.
   * Initialization that has to be done at the end of the process should occure within this method.
   */
  protected abstract void finalizeInitialization();

  /**
   * Create the command group that is shown as buttons on the bottom of the view
   *
   * @return the command group
   */
  @NotNull
  protected abstract CommandGroup createDefaultCommandGroup();

  /**
   * Initializes the table
   */
  protected void initTable() {
    TableModel tableModel = getTableModel();
    table = TableUtils.createStandardSortableTable( tableModel );
    table.setAutoResizeMode( JTable.AUTO_RESIZE_SUBSEQUENT_COLUMNS );
    table.setSelectionMode( ListSelectionModel.SINGLE_SELECTION );

    installCellRenderers();

    if ( tableModel instanceof BaseTableModel ) {
      if ( ( ( BaseTableModel ) tableModel ).hasRowNumbers() ) {
        //The first column is the row index
        table.getColumnModel().getColumn( 0 ).setPreferredWidth( 20 );
        table.getColumnModel().getColumn( 0 ).setMaxWidth( 30 );
      }
    }

    //Table popup Listener
    table.addMouseListener( new PopupMenuMouseListener() {
      @Override
      protected JPopupMenu getPopupMenu() {
        return showTablePopup();
      }
    } );

    table.addMouseListener( new MouseAdapter() {
      @Override
      public void mouseClicked( MouseEvent e ) {
        if ( e.getClickCount() == 2 ) {
          T selected = getSelectedBean();
          if ( selected == null ) {
            return;
          }
          openBean( selected );
        }
      }
    } );
  }

  /**
   * Open the bean.
   *
   * @param bean the bean that shall be opened
   */
  protected abstract void openBean( @NotNull T bean );

  /**
   * Returns the currently selected bean (if one is selected)
   *
   * @return the selected bean
   */
  @Nullable
  public abstract T getSelectedBean();

  /**
   * Install the cell renderes.
   */
  protected void installCellRenderers() {
    TableUtils.installDefaultRenderers( table );
    table.setDefaultRenderer( Enum.class, new EnumCellRenderer() );
    table.setDefaultRenderer( Rectangle2D.class, new Rectangle2DTableCellRenderer() );
    table.setDefaultRenderer( Rectangle2D.Double.class, new Rectangle2DTableCellRenderer() );
  }

  @Nullable
  protected abstract JPopupMenu showTablePopup();

  private boolean changing;

  /**
   * Initializes the selection change listeners
   */
  protected void initSelectionChanges() {
    //Update the selection
    SelectionManager selectionManager = SelectionManager.getInstance();

    /*
    * Update the selection model, when the selection state for this window changed
    */
    selectionManager.getWindowSelectionManager( getContext().getWindow() ).addSelectionListener( getBeanType(), new SelectionManager.SelectionListener<T>() {
      public void notifySelectionChanged( @NotNull Selection<T> selection ) {
        if ( changing ) {
          return;
        }
        changing = true;

        if ( selection.isSingleSelection() ) {
          int selectionIndex = getBeanIndex( selection.getSingleSelection() );
          getTable().getSelectionModel().setSelectionInterval( selectionIndex, selectionIndex );
        } else {
          getTable().getSelectionModel().clearSelection();
        }

        changing = false;
      }
    } );

    /**
     * Whenever the selection state of the table changes --> update the selection manager
     */
    getTable().getSelectionModel().addListSelectionListener( new ListSelectionListener() {
      @Override
      public void valueChanged( @NotNull ListSelectionEvent e ) {
        if ( e.getValueIsAdjusting() ) {
          return;
        }
        if ( changing ) {
          return;
        }
        changing = true;
        updateSelectionManager( getSelectedBean() );
        changing = false;
      }
    } );
  }

  /**
   * update the selection manager
   *
   * @param selectedBean the newly selected bean for the manager
   */
  public void updateSelectionManager( @Nullable T selectedBean ) {
    SelectionManager selectionManager = SelectionManager.getInstance();
    selectionManager.getWindowSelectionManager( getContext().getWindow() ).setSelection( new DefaultSelection<T>( getBeanType(), selectedBean ) );
  }

  protected boolean isSelectionNotEmpty() {
    return getTable().getSelectionModel().getMinSelectionIndex() > -1;
  }

  /**
   * Retrusnt the bean index of the given bean
   *
   * @param bean the bean the index for is requested
   * @return the index
   */
  protected abstract int getBeanIndex( @NotNull T bean );

  /**
   * Returns the type of the beans that are shown within this view
   *
   * @return the type of the beans
   */
  @NotNull
  protected abstract Class<T> getBeanType();

  /**
   * Returns the table model
   *
   * @return the table model
   */
  @NotNull
  public abstract TableModel getTableModel();

  /**
   * Returns the table
   *
   * @return the table
   */
  @NotNull
  public JTable getTable() {
    if ( this.table == null ) {
      throw new IllegalStateException( "Table is not initialized" );
    }
    return this.table;
  }
}
